import { Directive, Self, Host, Input } from '@angular/core';
import { VirtualScroll, Content } from 'ionic-angular';
var ListItemsAnimDirective = (function () {
    function ListItemsAnimDirective(virtualScroll, content) {
        this.virtualScroll = virtualScroll;
        this.content = content;
        this.sb = [];
        this.sb.push(content.ionScroll.subscribe(this.onScroll.bind(this)));
        this.sb.push(content.ionScrollEnd.subscribe(this.onScrollEnd.bind(this)));
        this.original_renderVirtual = virtualScroll.renderVirtual;
        virtualScroll.renderVirtual = this.overrodeRenderVirtual.bind(this);
    }
    Object.defineProperty(ListItemsAnimDirective.prototype, "options", {
        get: function () {
            return this._options;
        },
        set: function (v) {
            this._options = v;
        },
        enumerable: true,
        configurable: true
    });
    ListItemsAnimDirective.prototype.ngOnDestroy = function () {
        this.sb.forEach(function (a) { return a && a.unsubscribe && a.unsubscribe(); });
        this.sb = [];
    };
    ListItemsAnimDirective.prototype.overrodeRenderVirtual = function (needClean) {
        var _this = this;
        this.original_renderVirtual.apply(this.virtualScroll, [needClean]);
        this.intervalHandle = setInterval(function () {
            _this.updateItems();
        }, 200);
    };
    ListItemsAnimDirective.prototype.onScroll = function () {
        // https://github.com/ionic-team/ionic/blob/master/src/components/virtual-scroll/virtual-scroll.ts#L721
        if (this.virtualScroll._queue !== 1 /* ScrollQueue.NoChanges */ &&
            this.virtualScroll._queue !== 2 /* ScrollQueue.ChangeDetection */) {
            this.updateItems();
        }
    };
    ListItemsAnimDirective.prototype.onScrollEnd = function () {
        var _this = this;
        setTimeout(function () {
            _this.updateItems();
        });
    };
    ListItemsAnimDirective.prototype.updateItems = function () {
        if (this.locked) {
            this.nextCall = true;
        }
        else {
            this.nextCall = false;
            this.locked = true;
            this.updateItemsInternal();
        }
    };
    ListItemsAnimDirective.prototype.updateItemsInternal = function () {
        var isScrollingUp = this.lastScrollTop > this.content.scrollTop;
        this.lastScrollTop = this.content.scrollTop;
        var newVisibleItemIndex = 0;
        var newInvisibleItemIndex = 0;
        var nodes = this.virtualScroll._nodes.sort(function (a, b) { return (a.cell > b.cell ? 1 : -1) * (isScrollingUp ? -1 : 1); });
        var cells = this.virtualScroll._cells;
        var len = nodes.length;
        if (len && this.intervalHandle) {
            clearInterval(this.intervalHandle);
            this.intervalHandle = null;
        }
        for (var i = 0; i < len; i++) {
            var node = nodes[i];
            var cell = cells[node.cell];
            var inViewport = this.isInViewport(node, cell);
            if (inViewport) {
                if (node.wasInViewport) {
                    // const element = this.getElement(node);
                    // element.classList.remove('anim-listitem-exit');
                }
                else {
                    this.animEnter(node, cell, newVisibleItemIndex, isScrollingUp);
                    newVisibleItemIndex++;
                }
            }
            else {
                if (node.wasInViewport) {
                    this.animExit(node, cell, newInvisibleItemIndex, isScrollingUp);
                    newInvisibleItemIndex++;
                }
                else {
                    var element = this.getElement(node);
                    element.classList.remove('anim-listitem-visible');
                }
            }
            node.wasInViewport = inViewport;
        }
        this.locked = false;
        if (this.nextCall) {
            this.updateItems();
        }
    };
    ListItemsAnimDirective.prototype.isInViewport = function (node, cell) {
        if (!this.viewport) {
            this.viewport = this.content._scrollContent.nativeElement;
        }
        if ((this.viewport.clientTop + (this.options && this.options.offsetTop ? this.options.offsetTop : 0)) < (cell.top + cell.height - this.viewport.scrollTop) &&
            (this.viewport.clientTop + this.viewport.clientHeight - (this.options && this.options.offsetBottom ? this.options.offsetBottom : 0)) > (cell.top - this.viewport.scrollTop)) {
            return true;
        }
        return false;
    };
    ListItemsAnimDirective.prototype.animEnter = function (node, cell, newVisibleItemIndex, isScrollingUp) {
        var element = this.getElement(node);
        element.classList.remove("anim-listitem-" + node._prevIndex);
        node._prevIndex = newVisibleItemIndex;
        element.classList.add('anim-listitem-' + newVisibleItemIndex);
        element.classList.add('anim-listitem-visible');
        element.classList.remove('anim-listitem-hidden-top');
        element.classList.remove('anim-listitem-hidden-bottom');
    };
    ListItemsAnimDirective.prototype.animExit = function (node, cell, newVisibleItemIndex, isScrollingUp) {
        var element = this.getElement(node);
        element.classList.remove('anim-listitem-hidden-top');
        element.classList.remove('anim-listitem-hidden-bottom');
        element.classList.remove('anim-listitem-visible');
        element.classList.add("anim-listitem-hidden-" + (isScrollingUp ? 'top' : 'bottom'));
        element.classList.remove("anim-listitem-" + node._prevIndex);
        node._prevIndex = newVisibleItemIndex;
        element.classList.add('anim-listitem-' + newVisibleItemIndex);
    };
    ListItemsAnimDirective.prototype.getElement = function (node) {
        var rootNodes = node.view.rootNodes;
        for (var i = 0; i < rootNodes.length; i++) {
            if (rootNodes[i].nodeType === 1) {
                return rootNodes[i];
            }
        }
        return null;
    };
    ListItemsAnimDirective.decorators = [
        { type: Directive, args: [{ selector: '[listItemsAnim]' },] },
    ];
    /** @nocollapse */
    ListItemsAnimDirective.ctorParameters = function () { return [
        { type: VirtualScroll, decorators: [{ type: Self }, { type: Host },] },
        { type: Content, decorators: [{ type: Host },] },
    ]; };
    ListItemsAnimDirective.propDecorators = {
        "options": [{ type: Input, args: ['listItemsAnim',] },],
    };
    return ListItemsAnimDirective;
}());
export { ListItemsAnimDirective };
//# sourceMappingURL=ListItemsAnim.js.map